from types import NoneType


JSON_SUPPORTED_TYPES = [str, int, float, bool, NoneType]


def formatJsonLikeObj(obj: tuple | dict | list):
    """DEPRECATED/NOT NEEDED! Experimental object expander

    Args:
        obj (tuple | dict | list): The object to expand

    Returns:
        list[str]: The expanded object
    """
    st = str(obj)
    if len(st) < 20:
        return [st]

    new = []
    line = ""
    indents = 0
    type_symbol = False  # I mean this: < or >; <=True and >=False
    comma_newline = True
    for c in st:
        if c in "({[":
            if not type_symbol:
                indents += 1
                if line != "":
                    new.append(("    " * (indents - 1)) + line)
                new.append(("    " * (indents - 1) + c))
                line = ""
            else:
                line += c
            continue
        elif c in "]})":
            if not type_symbol:
                new.append("    " * indents + line)
                line = ""
                indents -= 1
            comma_newline = False
        if c == "<":
            type_symbol = True
        elif c == ">":
            type_symbol = False
        line += c
        if c == "," and comma_newline:
            new.append("    " * indents + line)
            line = ""
        comma_newline = True
    if line != "":
        new.append(line)

    return new


def replacePythonObjInJsonLike(obj: list | dict | tuple):
    result = obj.copy()
    del obj

    def iterate_list(l):
        for index, item in enumerate(l):
            if type(item) in [list, tuple]:
                iterate_list(item)
            elif type(item) == dict:
                iterate_dict(item)
            elif not type(item) in JSON_SUPPORTED_TYPES:
                l[index] = str(item)

    def iterate_dict(d):
        for key in d:
            if type(d[key]) in [list, tuple]:
                iterate_list(d[key])
            elif type(d[key]) == dict:
                iterate_dict(d[key])
            elif not type(d[key]) in JSON_SUPPORTED_TYPES:
                d[key] = str(d[key])

    if type(result) == dict:
        iterate_dict(result)
    else:
        iterate_list(result)

    return result



from typing import TypedDict, Type
from textwrap import wrap
from rich import print
from enum import Enum

from .filesystem_handlers import BaseFilesystemHandler
from .util import *

import threading
import datetime
import json
import re


class LoggingLevel(Enum):
    DEBUG = 0
    INFO = 1
    WARNING = 2
    ERROR = 3
    CRITICAL = 4


class ColorFormat(TypedDict):
    LoggingLevel.DEBUG: str
    LoggingLevel.INFO: str
    LoggingLevel.WARNING: str
    LoggingLevel.ERROR: str
    LoggingLevel.CRITICAL: str


class Logger:
    def __init__(
        self,
        logfile_path: str = None,
        show_threads: bool = True,
        level: LoggingLevel = LoggingLevel.INFO,
        colors: ColorFormat = {
            LoggingLevel.DEBUG: "#c7dcff",
            LoggingLevel.INFO: "#3caae6",
            LoggingLevel.WARNING: "#e0c528",
            LoggingLevel.ERROR: "#e61c5b",
            LoggingLevel.CRITICAL: "#f50a19",
        },
        time_format_file: str = "%d.%m.%Y %H:%M:%S",
        time_format_console: str = "%H:%M:%S",
        filesystem_handler: Type[BaseFilesystemHandler] = BaseFilesystemHandler(
        ),
    ) -> None:
        """Initialize a logger

        Args:
            logfile_path (str, optional): The path to the logfile. If none, no logs will be written to the filesystem. Defaults to None.
            show_threads (bool, optional): Show the threads in the console. Defaults to True.
            level (LoggingLevel, optional): The minimum logging level to be displayed in the console. Defaults to LoggingLevel.INFO.
            colors (TypedDict, optional): The colors to use for the console. Supports all colors rich is capable of. Defaults to { LoggingLevel.DEBUG: "#c7dcff", LoggingLevel.INFO: "#3caae6", LoggingLevel.WARNING: "#e0c528", LoggingLevel.ERROR: "#e61c5b", LoggingLevel.CRITICAL: "#f50a19", }.
            time_format_file (str, optional): The time format that is present in the log file. Defaults to "%d.%m.%Y %H:%M:%S".
            time_format_console (str, optional): The time format that is present in the console. Defaults to "%H:%M:%S".
            filesystem_handler (Type[BaseFilesystemHandler], optional): The filesystem handler. Defaults to BaseFilesystemHandler().
        """
        self.logfile = logfile_path
        self.show_threads = show_threads
        self.log_level = level
        self.colors = colors
        self.time_format_file = time_format_file
        self.time_format_console = time_format_console

        # File system handler
        self.filesystem_handler = filesystem_handler
        status, msg = self.filesystem_handler.init(logfile_path)
        if status:
            self.__log(LoggingLevel[status], [msg])

    def __log(self, level: LoggingLevel, elements) -> None:
        # ---------------------------------------------------------------------------- #
        #                                   Get time                                   #
        # ---------------------------------------------------------------------------- #
        # https://stackoverflow.com/a/39079819
        timezone = str(datetime.datetime.now(
            datetime.timezone.utc).astimezone().tzinfo)
        timeStrConsole = (
            datetime.datetime.now().strftime(self.time_format_console) + " " + timezone
        )
        timeStrFile = (
            datetime.datetime.now().strftime(self.time_format_file) + " " + timezone
        )
        # ---------------------------------------------------------------------------- #
        #                                  Get thread                                  #
        # ---------------------------------------------------------------------------- #
        thread = threading.current_thread().name
        if thread.split("-")[0] == "Thread":
            threadStr = "Thr" + thread.split("-")[1].split(" ")[0]
        elif thread == "MainThread":
            threadStr = "MainThr"
        else:
            threadStr = thread

        # ---------------------------------------------------------------------------- #
        #                               'Render' elements                              #
        # ---------------------------------------------------------------------------- #
        elementLines = []
        for e in elements:
            if type(e) in [list, dict, tuple]:
                elementLines += json.dumps(
                    replacePythonObjInJsonLike(e), indent=4
                ).split("\n")
            else:
                elementLines += wrap(str(e), width=35)

        # ---------------------------------------------------------------------------- #
        #                          Write to file if requested                          #
        # ---------------------------------------------------------------------------- #
        if self.logfile:
            ln = f"{level.name: <10} {thread.split(' ')[0]: <12} {timeStrFile}: "
            space = len(ln)
            lines = []
            for i, e in enumerate(elementLines):
                if i == 0:
                    lines.append(ln + e)
                else:
                    lines.append((" " * space) + e)

            self.filesystem_handler.write("\n".join(lines) + "\n")

        # ---------------------------------------------------------------------------- #
        #                   Print to console if level is high enough                   #
        # ---------------------------------------------------------------------------- #
        if level.value >= self.log_level.value:
            consoleThreadStr = (
                f"{threadStr: <8} " if self.show_threads else "")
            lvl = f"[{self.colors[level]}]{level.name: <10}[/{self.colors[level]}]"
            ln = f"[bold]{lvl} {consoleThreadStr}{timeStrConsole}:[/bold] "
            space = len(re.sub(r"\[[^\]]+\]", "", ln))
            lines = []
            for i, e in enumerate(elementLines):
                if i == 0:
                    lines.append(ln + e)
                else:
                    lines.append((" " * space) + e)
            print("\n".join(lines))

    def debug(self, *elements) -> None:
        self.__log(LoggingLevel.DEBUG, elements)

    def info(self, *elements) -> None:
        self.__log(LoggingLevel.INFO, elements)

    def warn(self, *elements) -> None:
        self.__log(LoggingLevel.WARNING, elements)

    def error(self, *elements) -> None:
        self.__log(LoggingLevel.ERROR, elements)

    def critical(self, *elements) -> None:
        self.__log(LoggingLevel.CRITICAL, elements)

    def __repr__(self) -> str:
        colors = {}
        for key, value in self.colors.items():
            colors[key.name] = f"[{value}]" + value + f"[/{value}]"

        cfg = {"colors": colors, "filesystem_handler": str(self.filesystem_handler),
               "level": str(self.log_level), "file": self.logfile}
        return "Logger " + json.dumps(cfg).replace("\"", "")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slofop']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'slofop',
    'version': '0.2.0',
    'description': '',
    'long_description': '# slofop\n\n## The hell is this name?\n\nThe name \'slofop\' stands for \'**S**imple **lo**gger **fo**r **P**ython\'.\n\n## Showcase\n\nAfter running the example.py file, ...\n\n - ...the terminal output is\n \n![Example script output](readme_assets/example_screenshot.png)\n\n - ...the content of \'test.log\' is\n```log\nDEBUG      MainThread   25.10.2022 17:00:49 CEST: {\n                                                      "__name__": "__main__",\n                                                      "__doc__": null,\n                                                      "__package__": null,\n                                                      "__loader__": "<_frozen_importlib_external.SourceFileLoader object at 0x7fab7107d510>",\n                                                      "__spec__": null,\n                                                      "__annotations__": {},\n                                                      "__builtins__": "<module \'builtins\' (built-in)>",\n                                                      "__file__": "/run/media/paddecraft/4EE0F299E0F2870D/Dev/PythonScripting/slofop/example.py",\n                                                      "__cached__": null,\n                                                      "filesystem_handlers": "<module \'slofop.filesystem_handlers\' from \'/run/media/paddecraft/4EE0F299E0F2870D/Dev/PythonScripting/slofop/slofop/filesystem_handlers.py\'>",\n                                                      "util": "<module \'slofop.util\' from \'/run/media/paddecraft/4EE0F299E0F2870D/Dev/PythonScripting/slofop/slofop/util.py\'>",\n                                                      "slofop": "<module \'slofop.slofop\' from \'/run/media/paddecraft/4EE0F299E0F2870D/Dev/PythonScripting/slofop/slofop/slofop.py\'>",\n                                                      "Logger": "<class \'slofop.slofop.Logger\'>",\n                                                      "LoggingLevel": "<enum \'LoggingLevel\'>",\n                                                      "os": "<module \'os\' from \'/usr/lib/python3.10/os.py\'>",\n                                                      "time": "<module \'time\' (built-in)>",\n                                                      "atexit": "<module \'atexit\' (built-in)>",\n                                                      "pathlib": "<module \'pathlib\' from \'/usr/lib/python3.10/pathlib.py\'>",\n                                                      "threading": "<module \'threading\' from \'/usr/lib/python3.10/threading.py\'>",\n                                                      "NoneType": "<class \'NoneType\'>",\n                                                      "BaseFilesystemHandler": "<class \'slofop.filesystem_handlers.BaseFilesystemHandler\'>",\n                                                      "TimedFilesystemHandler": "<class \'slofop.filesystem_handlers.TimedFilesystemHandler\'>",\n                                                      "log": "Logger {colors: {DEBUG: [#c7dcff]#c7dcff[/#c7dcff], INFO: [#3caae6]#3caae6[/#3caae6], WARNING: [#e0c528]#e0c528[/#e0c528], ERROR: [#e61c5b]#e61c5b[/#e61c5b], CRITICAL: [#f50a19]#f50a19[/#f50a19]}, filesystem_handler: FilesystemHandler(BaseFilesystemHandler), level: LoggingLevel.DEBUG, file: ./test.log}"\n                                                  }\nINFO       MainThread   25.10.2022 17:00:49 CEST: Info\nWARNING    MainThread   25.10.2022 17:00:49 CEST: Warn\nERROR      MainThread   25.10.2022 17:00:49 CEST: Error\nCRITICAL   MainThread   25.10.2022 17:00:49 CEST: Critical\nDEBUG      Thread-1     25.10.2022 17:00:49 CEST: Running in Thread-1 (testThread)\nINFO       Thread-1     25.10.2022 17:00:49 CEST: Thread running time 1\nDEBUG      Thread-1     25.10.2022 17:00:50 CEST: Running in Thread-1 (testThread)\nINFO       Thread-1     25.10.2022 17:00:50 CEST: Thread running time 2\nDEBUG      Thread-1     25.10.2022 17:00:51 CEST: Running in Thread-1 (testThread)\nINFO       Thread-1     25.10.2022 17:00:51 CEST: Thread running time 3\nDEBUG      Thread-1     25.10.2022 17:00:52 CEST: Running in Thread-1 (testThread)\nINFO       Thread-1     25.10.2022 17:00:52 CEST: Thread running time 4\n```\n\n## Class tree\n\n```py\nclass slofop.Logger:\n    def __init__(\n        self,\n        logfile_path: str = None,\n        show_threads: bool = True,\n        level: LoggingLevel = LoggingLevel.INFO,\n        colors: ColorFormat = {\n            LoggingLevel.DEBUG: "#c7dcff",\n            LoggingLevel.INFO: "#3caae6",\n            LoggingLevel.WARNING: "#e0c528",\n            LoggingLevel.ERROR: "#e61c5b",\n            LoggingLevel.CRITICAL: "#f50a19",\n        },\n        time_format_file: str = "%d.%m.%Y %H:%M:%S",\n        time_format_console: str = "%H:%M:%S",\n        filesystem_handler: Type[BaseFilesystemHandler] = BaseFilesystemHandler(\n        ),\n    ) -> None\n\n    def debug(*elements) -> None\n    def info(*elements) -> None\n    def warn(*elements) -> None\n    def error(*elements) -> None\n    def critical(*elements) -> None\n```',
    'author': 'PaddeCraft',
    'author_email': 'paddecraft@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/PaddeCraft/slofop',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import os
import time
import atexit
import pathlib
import threading

from types import NoneType


class BaseFilesystemHandler:
    def __init__(self) -> None:
        pass

    def init(self, path: str | NoneType) -> None | str:
        if not path:
            self.enabled = False
            return None, None
        self.logfile = path
        dir = os.path.split(path)[0]
        try:
            pathlib.Path(dir).mkdir(parents=True, exist_ok=True)
            self.enabled = True
            return None, None
        except PermissionError:
            self.enabled = False
            return (
                "WARNING",
                "Not enough permission to create root directory for the logs, logs will not be written to disk.",
            )

    def write(self, log: str):
        if self.enabled:
            with open(self.logfile, "a+", encoding="UTF-8") as lf:
                lf.write(log)

    def __repr__(self) -> str:
        return f"FilesystemHandler({self.__class__.__name__})"


class TimedFilesystemHandler(BaseFilesystemHandler):
    def __init__(self, interval: int = 1) -> None:
        """Write logs to disk every x seconds

        Args:
            interval (int, optional): The interval between writes. Defaults to 1.
        """
        self.logs = []
        self.interval = interval
        threading.Thread(target=self.thread, args=(self,))
        atexit.register(self.__write)

    def __write(self):
        current_logs = []
        [self.logs, current_logs] = [current_logs, self.logs]
        if self.enabled:
            with open(self.logfile, "a+", encoding="UTF-8") as lf:
                lf.write("".join(current_logs))

    def thread(self):
        while True:
            self.__write()
            time.sleep(self.interval)

    def write(self, log: str):
        self.logs.append(log)
